#Camera device
import os

import base64
import apogeeUSB
import Image
import time

EXTN=".png"


_IMAGE_SIZE=(1536, 1024)
_NUM_PIXELS=_IMAGE_SIZE[0]*_IMAGE_SIZE[1]
_TIME_WAIT_IMAGE_READY=3.0       #time to wait for the image to be ready
_TIME_POLL_IMAGE_READY=0.1       #time to wait for the image to be ready

_STR_LEN=256
_PROP_READ_FUNC="read_"
_PROP_WRITE_FUNC="write_"

_PROP_FAN_MODE="FanMode"
_PROP_IMAGING_STATUS="ImagingStatus"
_PROP_COOLER_BACK_OFF_POINT="CoolerBackoffPoint"
_PROP_COOLER_DRIVE="CoolerDrive"
_PROP_COOLER_ENABLE="CoolerEnable"
_PROP_COOLER_SET_POINT="CoolerSetPoint"
_PROP_COOLER_STATUS="CoolerStatus"
_PROP_TEMP_CCD="TempCCD"
_PROP_TEMP_HEAT_SINK="TempHeatsink"
_PROP_ROI_BINNING_H="RoiBinningH" 
_PROP_ROI_BINNING_V="RoiBinningV"
_PROP_ROI_PIXELS_H="RoiPixelsH"
_PROP_ROI_PIXELS_V="RoiPixelsV" 
_PROP_ROI_START_X="RoiStartX" 
_PROP_ROI_START_Y="RoiStartY"
_PROP_MAX_BINNNING_H="MaxBinningH"
_PROP_MAX_BINNNING_H="MaxBinningV"
_PROP_MAX_EXPOSURE_TIME="MaxExposureTime"
_PROP_DATA_BITS="DataBits" 

class Camera():
    
    def __init__(self):
        self.imageStr=None
        self.imageSize=_IMAGE_SIZE
        self.numPixels=_NUM_PIXELS
        self.setup()

    def setup(self):
        "Setup the camera"
                
        self.cam=apogeeUSB.CApnCamera()
        self.cam.InitDriver(1,0,0)
        
        #Do a system reset to ensure known state, flushing enabled etc 
        self.cam.ResetSystem()
    
        # Single image per download 
        #self.cam.write_ImageCount(1)
    
        #Normal mode
        self.cam.write_CameraMode(0)
    
        #self.cam.write_TriggerNormalGroup(True)
    
        #
        #self.cam.read_ImagingStatus()
    
        #self.cam.write_SequenceBulkDownload(True)
        #Expose time, shutter
    
        #self.cam.PreStartExpose(12)
       # self.roiBinningH(1)
       # self.roiBinningV(1)

        #print "setupImage size", self.imageSize
    
    def shutdown(self):
        "Shutdown the camera"
        self.cam.CloseDriver()

    def expose(self, exposureTime=1.0, light=True):
        "Expose to take a image and save to a file if a file is given"
            
        #self.cam.PreStartExpose(16)
        self.cam.Expose(exposureTime, light)
        #self.cam.PostStopExposure(False)
        
    def getImage(self):
        "Get a image"

        #Calculate image size
        self._calcuateDownloadImageSize()
                
        #Create a array of unsigned shorts
        imageUnshortArray=apogeeUSB.ushortArray(self.numPixels)
                                   
        count=_TIME_WAIT_IMAGE_READY/_TIME_POLL_IMAGE_READY
        while self.cam.read_ImagingStatus() !=3 or count==0:
            time.sleep(_TIME_POLL_IMAGE_READY)
            count-1
            
        if count==0:
            raise _CameraError("Camera image never ready")
            
        #Get image into a array
        self.cam.GetImage(imageUnshortArray)
        #=self.cam.GetImageData(imageUnshortArray)
        
        #Convert array into a string to be used by PIL to convet to an image
        self.imageStr = apogeeUSB.cdata(imageUnshortArray,self.numPixels*2)        
        
                                    
        #Conversion to list then image
        #self._slowConvert(imageUnshortArray)
        
    def saveImage(self, fileName):
        "Save last exposured image to a file"
        
        #Convert to string I 32 bit gray scale
        #"I;16" 16 bits
        img=Image.fromstring("I", self.imageSize, self.imageStr, "raw", "I;16", 0,1)
        
        #Write out image to file
        if fileName[-3:] != EXTN:
            fileName+=EXTN
        img.save("images/"+fileName)

    def imagingStatus(self):
        return self._propertyReadWrite(_PROP_IMAGING_STATUS)
    
    def _calcuateDownloadImageSize(self):
        "Determine the image size"
        sizeX=self.roiPixelsH()
        sizeY=self.roiPixelsV()
        self.imageSize=(sizeX, sizeY)
        self.numPixels=sizeX*sizeY
        
    def serialNumber(self):
        "Camera serial number"
        tpl=self.cam.GetCameraSerialNumber(_STR_LEN)
        #success 
        if tpl[0]==0:
            return tpl[1]
        else:
            return ""
    
    def fanSpeed(self, *speed):
        "Fan speed, 0 off, 3 max speed"
        return self._propertyReadWrite(_PROP_FAN_MODE, *speed)

    def coolerEnable(self, *enable):    
        "Cooler enable"
        return self._propertyReadWrite(_PROP_COOLER_ENABLE, *enable)        
        
    def coolerSetPoint(self, *setPoint):    
        "Cooler set point"
        return self._propertyReadWrite(_PROP_COOLER_SET_POINT, *setPoint)        
    
    def coolerBackOffPoint(self, *backOffPoint):    
        "Cooler back off point"
        return self._propertyReadWrite(_PROP_COOLER_BACK_OFF_POINT, *backOffPoint)        

    def coolerStatus(self, *status):    
        "Cooler status"
        return self._propertyReadWrite(_PROP_COOLER_STATUS, *status)        
    
    def coolerDrive(self, *level):    
        "Cooler drive level"
        return self._propertyReadWrite(_PROP_COOLER_DRIVE, *level)        
    
    def temperatureCCD(self, *temperature):    
        "CCD Temperature"
        return self._propertyReadWrite(_PROP_TEMP_CCD, *temperature)        

    def temperatureHeatSink(self, *temperature):    
        "Heat Sink Temperature"
        return self._propertyReadWrite(_PROP_TEMP_HEAT_SINK, *temperature)        
    
    def roiBinningH(self, *binning):    
        "ROI binning horizontal"
        return self._propertyReadWrite(_PROP_ROI_BINNING_H, *binning)        
    
    def roiBinningV(self, *binning):    
        "ROI binning vertical"
        return self._propertyReadWrite(_PROP_ROI_BINNING_V, *binning)        
    
    def roiPixelsH(self, *pixels):    
        "ROI pixels horizontal"
        return self._propertyReadWrite(_PROP_ROI_PIXELS_H, *pixels)        
    
    def roiPixelsV(self, *pixels):    
        "ROI pixels vertical"
        return self._propertyReadWrite(_PROP_ROI_PIXELS_V, *pixels)        

    def roiStartX(self, *start):    
        "ROI start X"
        return self._propertyReadWrite(_PROP_ROI_START_X, *start)        
    
    def roiStartY(self, *start):    
        "ROI start Y"
        return self._propertyReadWrite(_PROP_ROI_START_Y, *start)        

    def maxBinningH(self, *max):    
        "Maximum binning horizontal"
        return self._propertyReadWrite(_PROP_MAX_BINNNING_H, *max)        
    
    def maxBinningV(self, *max):    
        "Maximum binning vertical"
        return self._propertyReadWrite(_PROP_MAX_BINNNING_V, *max)      
      
    def maxExposureTime(self, *time):    
        "Maximum exposure Time"
        return self._propertyReadWrite(_PROP_MAX_EXPOSURE_TIME, *time)        
    
    def dataBits(self, *dataBits):    
        "Data bits 0: for 16 bits, 1 for 12 bits"
        return self._propertyReadWrite(_PROP_DATA_BITS, *dataBits)        

    def digitizationSpeed(self, *digSpeed):    
        """Digitization Speed: 
        0    10 Mhz
        1    5 Mhz
        2    2.5 Mhz
        3    1.25 Mhz
        4    0.612 Mhz
        5    0.306 Mhz
        6    0.153 Mhz
        7    0.076 Mhz"""        
        return self._propertyReadWrite(_PROP_DIGITIZATION_SPEED, *digSpeed)        
    
    _PROP_DATA_BITS
    def _propertyReadWrite(self, property, *value):
        """read and write helper function"""
        lenValue=len(value)
        if lenValue==0:
            getter=getattr(self.cam, _PROP_READ_FUNC+property)
            value=getter()
        elif lenValue==1:
            setter=getattr(self.cam, _PROP_WRITE_FUNC+property)
            setter(value[0])          
            
            getter=getattr(self.cam, _PROP_READ_FUNC+property)
            value=getter()
            
        else:
            raise _CameraError("Property write Only 1 value allowed")
            
        return value
        
    def _timeStamp(self):
        return time.strftime('%d-%b-%y_%X')
        
    def _slowConvert(self,  imageUnshortArray):
        "Slow conservative conversion for testing faster method"
        import array
        #Slow conservative convert
        listArray=[0]*_NUM_PIXELS
        for i in range(0, _NUM_PIXELS):
            listArray[i]=imageUnshortArray[i]
                
        #Convert list array to numpy array
        dataArray=array.array('I')
        dataArray.fromlist(listArray)
        #print dataArray.buffer_info()
        #imageArray=numpy.reshape(dataArray, _SIZE_IMAGE)

        img=Image.fromstring("I", _SIZE_IMAGE, dataArray.tostring())
        img.save("testCamArray"+".png")        

    
    def convertBase64(fileName):    
        """get a image data encoded as binary64"""
        import array
        imageFileName=fileName+EXTN     
    
        imageBinArray=array.array('B')
        imageFileSize=statinfo=os.stat(imageFileName).st_size
        
        #Read from file into binary array
        fh=open(imageFileName, 'rb')
        imageBinArray.fromfile(fh, imageFileSize)
        fh.close()
                  
        #encode array string into base 64
        imageEncodeStr=base64.standard_b64encode(imageBinArray.tostring())
          
        #print "imagesize ",len(imageEncodeStr), " imagesize encoded ",imageFileSize
            
        return imageEncodeStr

class _CameraError(Exception):
    def __init__(self, message):
        self.message=message      
    def __str__(self):
        return "Camera Error: "+self.message
    def __repr__(self):
        return "Camera Error: "+self.message

if __name__=="__main__":
    import time
    c=Camera()
    
    #Image binning
    binH=1
    c.roiBinningH(binH)
    c.roiPixelsH(1536/binH)
    
    binV=1
    #binV=4 does not work
    c.roiBinningV(binV)    
    c.roiPixelsV(1024/binV)
    
    c.expose(0.1)
    t1=time.time()
    c.getImage()
    t2=time.time()    
    c.saveImage("TestImage_"+c._timeStamp())
    
    print "time get image ", t2-t1
        
    #c.shutdown()
    
